from ..contract_generator import ContractGenerator


class AdminUpgradeabilityProxyGenerator(ContractGenerator):
    ARTIFACT_FILENAME = "AdminUpgradeabilityProxy.json"
    IMPLEMENTATION_SLOT = int('0x360894a13ba1a3210667c828492db98dca3e2076cc3735a920a3ca505d382bbc', 16)
    ADMIN_SLOT = int('0xb53127684a568b3173ae13b9f8a6016e243e63b6e8ee1178d6a717850b5d6103', 16)

    def __init__(self, logic_address: str, admin_address: str):
        super().__init__(self.ARTIFACT_FILENAME)
        self._setup(logic_address, admin_address)

    # private

    def _setup(self, logic_address: str, admin_address: str) -> None:
        self._write_address(self.IMPLEMENTATION_SLOT, logic_address)
        self._write_address(self.ADMIN_SLOT, admin_address)



from contracts.admin_upgradeability_proxy import check_admin_upgradeability_proxy
from contracts.community_locker import check_community_locker
from contracts.eth_erc20 import check_eth_erc20
from contracts.key_storage import check_key_storage
from contracts.message_proxy_for_schain import check_message_proxy_for_schain
from contracts.proxy_admin import check_proxy_admin
from contracts.token_manager_erc20 import check_token_manager_erc20
from contracts.token_manager_erc721 import check_token_manager_erc721
from contracts.token_manager_erc1155 import check_token_manager_erc1155
from contracts.token_manager_erc721_with_metadata import check_token_manager_erc721_with_metadata
from contracts.token_manager_eth import check_token_manager_eth
from contracts.token_manager_linker import check_token_manager_linker
import json


with open('config.json') as config_file:
    config = json.load(config_file)
    owner_address = config['schain_owner']
    schain_name = config['schain_name']
    eth_deposit_box = config['eth_deposit_box']
    erc20_deposit_box = config['erc20_deposit_box']
    erc721_deposit_box = config['erc721_deposit_box']
    erc1155_deposit_box = config['erc1155_deposit_box']
    linker_address = config['linker']
    community_pool = config['community_pool']
    erc721_with_metadata_deposit_box = config['erc721_with_metadata_deposit_box']


def main():
    check_proxy_admin(owner_address)
    check_admin_upgradeability_proxy()
    check_message_proxy_for_schain(owner_address, schain_name)
    check_key_storage(owner_address)
    check_community_locker(owner_address, schain_name, community_pool)
    check_token_manager_linker(owner_address, linker_address)
    check_token_manager_eth(owner_address, eth_deposit_box, schain_name)
    check_token_manager_erc20(owner_address, erc20_deposit_box, schain_name)
    check_token_manager_erc721(owner_address, erc721_deposit_box, schain_name)
    check_token_manager_erc1155(owner_address, erc1155_deposit_box, schain_name)
    check_token_manager_erc721_with_metadata(owner_address, erc721_with_metadata_deposit_box, schain_name)
    check_eth_erc20(owner_address)

    print('All tests pass')


if __name__ == '__main__':
    main()



from ..contract_generator import ContractGenerator


class ProxyAdminGenerator(ContractGenerator):
    ARTIFACT_FILENAME = "ProxyAdmin.json"

    def __init__(self, owner_address: str):
        super().__init__(self.ARTIFACT_FILENAME)
        self._setup(owner_address)

    # private

    def _setup(self, owner_address: str) -> None:
        self._write_address(0, owner_address)



from ..contract_generator import ContractGenerator


class KeyStorageGenerator(ContractGenerator):
    ARTIFACT_FILENAME = "KeyStorage.json"
    DEFAULT_ADMIN_ROLE = (0).to_bytes(32, 'big')

    # ---------- storage ----------
    # --------Initializable--------
    # 0:    _initialized, _initializing;
    # -----ContextUpgradeable------
    # 1:    __gap
    # ...   __gap
    # 50:   __gap
    # ------ERC165Upgradeable------
    # 51:   __gap
    # ...   __gap
    # 100:  __gap
    # --AccessControlUpgradeable---
    # 101:  _roles
    # 102:  __gap
    # ...   __gap
    # 150:  __gap
    # AccessControlEnumerableUpgradeable
    # 151:  _roleMembers
    # 152:  __gap
    # ...   __gap
    # 200:  __gap
    # ----------KeyStorage----------

    INITIALIZED_SLOT = 0
    ROLES_SLOT = 101
    ROLE_MEMBERS_SLOT = 151

    def __init__(self, deployer_address: str):
        super().__init__(self.ARTIFACT_FILENAME)
        self._setup(deployer_address)

    # private

    def _setup(self, deployer_address: str) -> None:
        self._write_uint256(self.INITIALIZED_SLOT, 1)
        self._setup_role(self.ROLES_SLOT, self.ROLE_MEMBERS_SLOT, self.DEFAULT_ADMIN_ROLE, [deployer_address])



import json
import os

from ima_predeployed.contracts.eth_erc20 import EthErc20Generator
from ima_predeployed.contracts.token_manager_erc20 import TokenManagerErc20Generator
from ima_predeployed.contracts.token_manager_erc721 import TokenManagerErc721Generator
from ima_predeployed.contracts.token_manager_erc1155 import TokenManagerErc1155Generator
from ima_predeployed.contracts.token_manager_erc721_with_metadata import TokenManagerErc721WithMetadataGenerator
from ima_predeployed.contracts.token_manager_eth import TokenManagerEthGenerator
from ima_predeployed.contracts.token_manager_linker import TokenManagerLinkerGenerator
from .contracts.community_locker import CommunityLockerGenerator
from .contract_generator import ContractGenerator
from .contracts.key_storage import KeyStorageGenerator
from .upgradeable_contract_generator import UpgradeableContractGenerator
from .contracts.proxy_admin import ProxyAdminGenerator
from .contracts.message_proxy_for_schain import MessageProxyForSchainGenerator
from .addresses import \
    PROXY_ADMIN_ADDRESS, \
    MESSAGE_PROXY_FOR_SCHAIN_ADDRESS, \
    MESSAGE_PROXY_FOR_SCHAIN_IMPLEMENTATION_ADDRESS, KEY_STORAGE_IMPLEMENTATION_ADDRESS, KEY_STORAGE_ADDRESS, \
    COMMUNITY_LOCKER_IMPLEMENTATION_ADDRESS, COMMUNITY_LOCKER_ADDRESS, \
    TOKEN_MANAGER_ERC1155_ADDRESS, TOKEN_MANAGER_ERC1155_IMPLEMENTATION_ADDRESS, TOKEN_MANAGER_LINKER_IMPLEMENTATION_ADDRESS, \
    TOKEN_MANAGER_LINKER_ADDRESS, TOKEN_MANAGER_ETH_IMPLEMENTATION_ADDRESS, TOKEN_MANAGER_ETH_ADDRESS, \
    TOKEN_MANAGER_ERC20_IMPLEMENTATION_ADDRESS, TOKEN_MANAGER_ERC20_ADDRESS, \
    TOKEN_MANAGER_ERC721_IMPLEMENTATION_ADDRESS, TOKEN_MANAGER_ERC721_ADDRESS, ETH_ERC20_IMPLEMENTATION_ADDRESS, \
    ETH_ERC20_ADDRESS, TOKEN_MANAGER_ERC721_WITH_METADATA_IMPLEMENTATION_ADDRESS, TOKEN_MANAGER_ERC721_WITH_METADATA_ADDRESS


def generate_contracts(
        owner_address: str,
        schain_name: str,
        contracts_on_mainnet: dict) -> dict:
    proxy_admin = ProxyAdminGenerator(owner_address)

    message_proxy_for_schain_implementation = ContractGenerator(MessageProxyForSchainGenerator.ARTIFACT_FILENAME)
    message_proxy_for_schain = UpgradeableContractGenerator(
        MESSAGE_PROXY_FOR_SCHAIN_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        MessageProxyForSchainGenerator(owner_address, schain_name))

    key_storage_implementation = ContractGenerator(KeyStorageGenerator.ARTIFACT_FILENAME)
    key_storage = UpgradeableContractGenerator(
        KEY_STORAGE_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        KeyStorageGenerator(owner_address))

    community_locker_implementation = ContractGenerator(CommunityLockerGenerator.ARTIFACT_FILENAME)
    community_locker = UpgradeableContractGenerator(
        COMMUNITY_LOCKER_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        CommunityLockerGenerator(owner_address, schain_name, contracts_on_mainnet['community_pool_address'])
    )

    token_manager_linker_implementation = ContractGenerator(TokenManagerLinkerGenerator.ARTIFACT_FILENAME)
    token_manager_linker = UpgradeableContractGenerator(
        TOKEN_MANAGER_LINKER_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerLinkerGenerator(owner_address, contracts_on_mainnet['linker_address'])
    )

    token_manager_eth_implementation = ContractGenerator(TokenManagerEthGenerator.ARTIFACT_FILENAME)
    token_manager_eth = UpgradeableContractGenerator(
        TOKEN_MANAGER_ETH_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerEthGenerator(owner_address, contracts_on_mainnet['deposit_box_eth_address'], schain_name)
    )

    token_manager_erc20_implementation = ContractGenerator(TokenManagerErc20Generator.ARTIFACT_FILENAME)
    token_manager_erc20 = UpgradeableContractGenerator(
        TOKEN_MANAGER_ERC20_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerErc20Generator(owner_address, contracts_on_mainnet['deposit_box_erc20_address'], schain_name)
    )

    token_manager_erc721_implementation = ContractGenerator(TokenManagerErc721Generator.ARTIFACT_FILENAME)
    token_manager_erc721 = UpgradeableContractGenerator(
        TOKEN_MANAGER_ERC721_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerErc721Generator(owner_address, contracts_on_mainnet['deposit_box_erc721_address'], schain_name)
    )

    token_manager_erc1155_implementation = ContractGenerator(TokenManagerErc1155Generator.ARTIFACT_FILENAME)
    token_manager_erc1155 = UpgradeableContractGenerator(
        TOKEN_MANAGER_ERC1155_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerErc1155Generator(owner_address, contracts_on_mainnet['deposit_box_erc1155_address'], schain_name)
    )

    token_manager_erc721_with_metadata_implementation = ContractGenerator(TokenManagerErc721WithMetadataGenerator.ARTIFACT_FILENAME)
    token_manager_erc721_with_metadata = UpgradeableContractGenerator(
        TOKEN_MANAGER_ERC721_WITH_METADATA_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        TokenManagerErc721WithMetadataGenerator(owner_address, contracts_on_mainnet['deposit_box_erc721_with_metadata_address'], schain_name)
    )

    eth_erc20_implementation = ContractGenerator(EthErc20Generator.ARTIFACT_FILENAME)
    eth_erc20 = UpgradeableContractGenerator(
        ETH_ERC20_IMPLEMENTATION_ADDRESS,
        PROXY_ADMIN_ADDRESS,
        EthErc20Generator(owner_address)
    )

    return {
        PROXY_ADMIN_ADDRESS: proxy_admin.generate_contract(),

        MESSAGE_PROXY_FOR_SCHAIN_ADDRESS: message_proxy_for_schain.generate_contract(),
        MESSAGE_PROXY_FOR_SCHAIN_IMPLEMENTATION_ADDRESS: message_proxy_for_schain_implementation.generate_contract(),

        KEY_STORAGE_ADDRESS: key_storage.generate_contract(),
        KEY_STORAGE_IMPLEMENTATION_ADDRESS: key_storage_implementation.generate_contract(),

        COMMUNITY_LOCKER_ADDRESS: community_locker.generate_contract(),
        COMMUNITY_LOCKER_IMPLEMENTATION_ADDRESS: community_locker_implementation.generate_contract(),

        TOKEN_MANAGER_LINKER_ADDRESS: token_manager_linker.generate_contract(),
        TOKEN_MANAGER_LINKER_IMPLEMENTATION_ADDRESS: token_manager_linker_implementation.generate_contract(),

        TOKEN_MANAGER_ETH_ADDRESS: token_manager_eth.generate_contract(),
        TOKEN_MANAGER_ETH_IMPLEMENTATION_ADDRESS: token_manager_eth_implementation.generate_contract(),

        TOKEN_MANAGER_ERC20_ADDRESS: token_manager_erc20.generate_contract(),
        TOKEN_MANAGER_ERC20_IMPLEMENTATION_ADDRESS: token_manager_erc20_implementation.generate_contract(),

        TOKEN_MANAGER_ERC721_ADDRESS: token_manager_erc721.generate_contract(),
        TOKEN_MANAGER_ERC721_IMPLEMENTATION_ADDRESS: token_manager_erc721_implementation.generate_contract(),

        TOKEN_MANAGER_ERC1155_ADDRESS: token_manager_erc1155.generate_contract(),
        TOKEN_MANAGER_ERC1155_IMPLEMENTATION_ADDRESS: token_manager_erc1155_implementation.generate_contract(),

        TOKEN_MANAGER_ERC721_WITH_METADATA_ADDRESS: token_manager_erc721_with_metadata.generate_contract(),
        TOKEN_MANAGER_ERC721_WITH_METADATA_IMPLEMENTATION_ADDRESS: token_manager_erc721_with_metadata_implementation.generate_contract(),

        ETH_ERC20_ADDRESS: eth_erc20.generate_contract(),
        ETH_ERC20_IMPLEMENTATION_ADDRESS: eth_erc20_implementation.generate_contract()
    }


def generate_abi_key(name: str, address: str, abi: list) -> dict:
    return {
        name + '_address': address,
        name + '_abi': abi
    }


def generate_abi() -> dict:
    return {
        **generate_abi_key(
            'proxy_admin',
            PROXY_ADMIN_ADDRESS,
            ContractGenerator(ProxyAdminGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'message_proxy_chain',
            MESSAGE_PROXY_FOR_SCHAIN_ADDRESS,
            ContractGenerator(MessageProxyForSchainGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'key_storage',
            KEY_STORAGE_ADDRESS,
            ContractGenerator(KeyStorageGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'community_locker',
            COMMUNITY_LOCKER_ADDRESS,
            ContractGenerator(CommunityLockerGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_linker',
            TOKEN_MANAGER_LINKER_ADDRESS,
            ContractGenerator(TokenManagerLinkerGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_eth',
            TOKEN_MANAGER_ETH_ADDRESS,
            ContractGenerator(TokenManagerEthGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_erc20',
            TOKEN_MANAGER_ERC20_ADDRESS,
            ContractGenerator(TokenManagerErc20Generator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_erc721',
            TOKEN_MANAGER_ERC721_ADDRESS,
            ContractGenerator(TokenManagerErc721Generator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_erc1155',
            TOKEN_MANAGER_ERC1155_ADDRESS,
            ContractGenerator(TokenManagerErc1155Generator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'token_manager_erc721_with_metadata',
            TOKEN_MANAGER_ERC721_WITH_METADATA_ADDRESS,
            ContractGenerator(TokenManagerErc721WithMetadataGenerator.ARTIFACT_FILENAME).abi),
        **generate_abi_key(
            'eth_erc20',
            ETH_ERC20_ADDRESS,
            ContractGenerator(EthErc20Generator.ARTIFACT_FILENAME).abi),
        **{
            'ERC20OnChain_abi': ContractGenerator('ERC20OnChain.json').abi,
            'ERC721OnChain_abi': ContractGenerator('ERC721OnChain.json').abi,
            'ERC1155OnChain_abi': ContractGenerator('ERC1155OnChain.json').abi
        }
    }


def main() -> None:
    contracts_dir = os.path.join(os.path.dirname(__file__), 'artifacts')
    proxy_admin_path = os.path.join(contracts_dir, 'ProxyAdmin.json')    
    with open(proxy_admin_path, encoding='utf-8') as fp:
        proxy_admin = json.load(fp)
        print(proxy_admin['contractName'])


if __name__ == '__main__':
    main()



from ima_predeployed.addresses import MESSAGE_PROXY_FOR_SCHAIN_ADDRESS, TOKEN_MANAGER_LINKER_ADDRESS, \
    COMMUNITY_LOCKER_ADDRESS
from ima_predeployed.contract_generator import ContractGenerator, next_slot
from web3 import Web3


class TokenManagerGenerator(ContractGenerator):
    ARTIFACT_FILENAME = "TokenManager.json"
    DEFAULT_ADMIN_ROLE = (0).to_bytes(32, 'big')
    AUTOMATIC_DEPLOY_ROLE = Web3.solidityKeccak(['string'], ['AUTOMATIC_DEPLOY_ROLE'])
    TOKEN_REGISTRAR_ROLE = Web3.solidityKeccak(['string'], ['TOKEN_REGISTRAR_ROLE'])

    # ---------- storage ----------
    # --------Initializable--------
    # 0:    _initialized, _initializing;
    # -----ContextUpgradeable------
    # 1:    __gap
    # ...   __gap
    # 50:   __gap
    # ------ERC165Upgradeable------
    # 51:   __gap
    # ...   __gap
    # 100:  __gap
    # --AccessControlUpgradeable---
    # 101:  _roles
    # 102:  __gap
    # ...   __gap
    # 150:  __gap
    # AccessControlEnumerableUpgradeable
    # 151:  _roleMembers
    # 152:  __gap
    # ...   __gap
    # 200:  __gap
    # ---------TokenManager---------
    # 201:  messageProxy
    # 202:  tokenManagerLinker
    # 203:  communityLocker
    # 204:  schainHash
    # 205:  depositBox, automaticDeploy
    # 206:  tokenManagers

    INITIALIZED_SLOT = 0
    ROLES_SLOT = 101
    ROLE_MEMBERS_SLOT = 151
    MESSAGE_PROXY_SLOT = 201
    TOKEN_MANAGER_LINKER_SLOT = next_slot(MESSAGE_PROXY_SLOT)
    COMMUNITY_LOCKER_SLOT = next_slot(TOKEN_MANAGER_LINKER_SLOT)
    SCHAIN_HASH_SLOT = next_slot(COMMUNITY_LOCKER_SLOT)
    DEPOSIT_BOX_SLOT = next_slot(SCHAIN_HASH_SLOT)
    AUTOMATIC_DEPLOY_SLOT = DEPOSIT_BOX_SLOT
    TOKEN_MANAGERS_SLOT = next_slot(AUTOMATIC_DEPLOY_SLOT)

    def __init__(self, deployer_address: str, deposit_box_address: str, schain_name: str):
        super().__init__(self.ARTIFACT_FILENAME)
        self._setup_token_manager(deployer_address, deposit_box_address, schain_name)

    # private

    def _setup_token_manager(self, deployer_address: str, deposit_box_address: str, schain_name: str) -> None:
        self._write_uint256(self.INITIALIZED_SLOT, 1)
        self._setup_role(self.ROLES_SLOT, self.ROLE_MEMBERS_SLOT, self.DEFAULT_ADMIN_ROLE, [deployer_address])
        self._setup_role(self.ROLES_SLOT, self.ROLE_MEMBERS_SLOT, self.AUTOMATIC_DEPLOY_ROLE, [deployer_address])
        self._setup_role(self.ROLES_SLOT, self.ROLE_MEMBERS_SLOT, self.TOKEN_REGISTRAR_ROLE, [deployer_address])
        self._write_address(self.MESSAGE_PROXY_SLOT, MESSAGE_PROXY_FOR_SCHAIN_ADDRESS)
        self._write_address(self.TOKEN_MANAGER_LINKER_SLOT, TOKEN_MANAGER_LINKER_ADDRESS)
        self._write_address(self.COMMUNITY_LOCKER_SLOT, COMMUNITY_LOCKER_ADDRESS)
        self._write_bytes32(self.SCHAIN_HASH_SLOT, Web3.solidityKeccak(['string'], [schain_name]))
        self._write_address(self.DEPOSIT_BOX_SLOT, deposit_box_address)



from .contract_generator import ContractGenerator
from .contracts.admin_upgradeability_proxy import AdminUpgradeabilityProxyGenerator


class UpgradeableContractGenerator(AdminUpgradeabilityProxyGenerator):
    def __init__(self, implementation_address: str, proxy_admin_address, implementation_generator: ContractGenerator):
        super().__init__(implementation_address, proxy_admin_address)
        self.implementation_generator = implementation_generator

    def generate_contract(self, balance: int = 0, nonce: int = 0) -> dict:
        contract = super().generate_contract(balance, nonce)
        contract['storage'].update(self.implementation_generator.generate_contract(balance, nonce)['storage'])
        return contract



from ima_predeployed.contracts.token_manager import TokenManagerGenerator


class TokenManagerErc721WithMetadataGenerator(TokenManagerGenerator):
    ARTIFACT_FILENAME = "TokenManagerERC721WithMetadata.json"

    # ---------- storage ----------
    # --------Initializable--------
    # 0:    _initialized, _initializing;
    # -----ContextUpgradeable------
    # 1:    __gap
    # ...   __gap
    # 50:   __gap
    # ------ERC165Upgradeable------
    # 51:   __gap
    # ...   __gap
    # 100:  __gap
    # --AccessControlUpgradeable---
    # 101:  _roles
    # 102:  __gap
    # ...   __gap
    # 150:  __gap
    # AccessControlEnumerableUpgradeable
    # 151:  _roleMembers
    # 152:  __gap
    # ...   __gap
    # 200:  __gap
    # ---------TokenManager---------
    # 201:  messageProxy
    # 202:  tokenManagerLinker
    # 203:  communityLocker
    # 204:  schainHash
    # 205:  depositBox, automaticDeploy
    # 206:  tokenManagers
    # ------TokenManagerERC721------
    # 207:  clonesErc721
    # ------TokenManagerERC721WithMetadata------

    def __init__(self, deployer_address: str, deposit_box_address: str, schain_name: str):
        super().__init__(deployer_address, deposit_box_address, schain_name)
